/* ECE243 Project
   Marcin Smigiel
   Patrick Judd
   
   Oct 16 2015
   Modified address for program to run on DE1-SoC
   changes made by Fred Aulich

   March 21, 2008

	Modified by Patrick on March 5 2011
	- New Lego controller and board configuration
		- changed device base addresses
		- new motor and sensor interface
	- Lite version 
		- due to new VGA interface I have removed the vga feature from the lite version
		- due to simpler functionality I have removed interrupts 
		
	- 2.1 (Patrick, Feb 6 2012)
		- Use software debouncing instead of max value approach
			- a bounce on one bit causes the note to change i.e. 0111 -> 0000 -> 0100 -> 0000

*/

/*
constant registers:

r10: sample data offset

For debouncing:
r22: debounce counter
r23: stable sensor data
r24: previous sensor data
*/

.include "waves.s"
/********************************* Data ***************************************/
	.data
	.align 2

	# device base addresses
	.EQU TIMER0, 			0xFF202000
	.EQU REDLEDS, 			0xFF200000
#	.EQU GREENLEDS,			0xFF200010
  	.EQU JP1, 				0xFF200060
	.EQU ADDR_AUDIODACFIFO, 0xFF203040
	.EQU SWITCHES,			0xFF200040
	
	# write these to JP1 to control the motors
	.EQU MOTOR_FWD, 		0xFFDFFFFE
	.EQU MOTOR_REV, 		0xFFDFFFFC
	.EQU MOTOR_OFF,			0xFFDFFFFF
	
	# sets the appropriate sensor select bit to 0
	# leaves a window of 0s for the threshold value to be written into
	# window = bit26 to bit23
	.EQU SENSOR0_THRESH_WINDOW,		0xF83FFBFF
	.EQU SENSOR1_THRESH_WINDOW,		0xF83FEFFF
	.EQU SENSOR2_THRESH_WINDOW,		0xF83FBFFF
	.EQU SENSOR3_THRESH_WINDOW,		0xF83EFFFF
	
	# sets the sensors into state mode
	.EQU SET_STATE_MODE,	0xFFDFFFFF
	
	# sets appropriate directions for lego controller
	.EQU SET_JP1_DIR,		0x07F557FF
	
	# threshold values for the sensors
	.EQU SENSOR0_THRESH,	0x8
	.EQU SENSOR1_THRESH,	0x8
	.EQU SENSOR2_THRESH,	0x8
	.EQU SENSOR3_THRESH,	0x8
	
	# how many iterations to wait for signal to debounce
	# I am using a lower value for zero so I can distinguish 2 consecutive notes
	.EQU DEBOUNCE_WAIT,		0x0F00
	.EQU ZERO_DEBOUNCE_WAIT, 0x0300

/***************************** Program Code ***********************************/
	.text
	.align 2
	.global _start
_start:

	/* INITIALIZATION OF SENSORS*/
	
	movia	r4, JP1
  	movia  r8, SET_JP1_DIR
  	stwio  r8, 4(r4)
	
	movi	r8, SENSOR0_THRESH
	movia	r9, SENSOR0_THRESH_WINDOW	
	slli	r8, r8, 23
	or		r8, r8, r9
	stwio	r8, 0(r4)
	
	movi	r8, SENSOR1_THRESH
	movia	r9, SENSOR1_THRESH_WINDOW	
	slli	r8, r8, 23
	or		r8, r8, r9
	stwio	r8, 0(r4)
	
	movi	r8, SENSOR2_THRESH
	movia	r9, SENSOR2_THRESH_WINDOW	
	slli	r8, r8, 23
	or		r8, r8, r9
	stwio	r8, 0(r4)
	
	movi	r8, SENSOR3_THRESH
	movia	r9, SENSOR3_THRESH_WINDOW	
	slli	r8, r8, 23
	or		r8, r8, r9
	stwio	r8, 0(r4)
	
	movia	r8, SET_STATE_MODE
	stwio	r8, 0(r4)


	/* MAIN PROGRAM */

MAINLOOP:

	# spin loop to wait for space in the FIFO
	movia 	r2, ADDR_AUDIODACFIFO
	ldwio	r2, 4(r2)				# get fifo space register
	srli	r2,r2,16				# shift over to bits 31-16

	andi	r2, r2, 0xFF			# isolate bits 7-0 (FIFO left)
	cmplti 	r2, r2, 0x20			# if (space in FIFO left < 0x20) branch to MAINLOOP
	bne		r2, r0, MAINLOOP 
	

	/* POLL SWITCHES */
	
	movia	r2, SWITCHES
	ldwio	r8, 0(r2)
	andi	r9, r8, 0x1 			# isolate switch 0
	bne		r9, r0, ON
	andi	r9, r8, 0x2				# isolate switch 1
	bne		r9, r0, NO_MOTOR
	
OFF:
	movia	r2, JP1
	movia	r3, MOTOR_OFF
	stwio	r3, 0(r2)
	movi	r23, 0x1				# set pitch = 1 (no sound)
	mov		r10, r0
	br 		NO_UPDATE				# turn off sound
	
ON:
	movia	r2, JP1
	movia	r3, MOTOR_FWD
	stwio	r3, 0(r2)
	br 		MOTOR_SET
	
NO_MOTOR:
	movia	r2, JP1					# turn motor off
	movia	r3, MOTOR_OFF
	stwio	r3, 0(r2)
	# but keep reading/playing notes
	
MOTOR_SET:
	
	# POLL SENSORS 
	movia		r2, JP1
	ldwio		r8, 0(r2)
	srli		r8, r8, 27
	andi		r8, r8, 0xF			# r8 = sensor data
	
	movia		r2, REDLEDS		# debug current sensor value
	stwio		r8, 0(r2)
	
	# DEBOUNCING
	# sets stable data to sensor data when sensor data has been constant for X iterations
	
	# r8 = sensor data
	# r24 = prev sensor data
	# r23 = stable data
	
	# if ( data != prev )
	#	prev = data
	#	reset counter
	# else
	#	if data == 0 && counter > ZERO_DEBOUNCE_WAIT
	#		reset sample offset (inserts audio blip)
	#	if counter > DEBOUNCE WAIT
	#		stable data = data
	#
	# counter++
	#

	beq			r8, r24, STABLE
	mov			r24, r8				# update prev sensor data
	mov			r22, r0 			# reset counter
	br			UNSTABLE
	
STABLE:
	bne			r24, r0, NOT_ZERO
	movi		r2, ZERO_DEBOUNCE_WAIT
	blt			r22, r2, NOT_ZERO
	mov			r10, r0
	
NOT_ZERO:
	movi		r2, DEBOUNCE_WAIT	
	blt			r22, r2, UNSTABLE	# if we are still waiting for debounce
	
	# SIGNAL IS STABLE
	mov			r23, r8				# stable data = sensor data
	
UNSTABLE:
	addi		r22,r22,1			# increment debounce counter
	
	# if (stable data != 0)
	#	note = stable data
	beq			r23, r0, NO_UPDATE	# don't update on 0000
	
CHANGE_NOTE:		
	mov			r7, r23				# copy pitch
	
NO_UPDATE:
	
	# debug: show current note on REDLEDS
	movia		r2, REDLEDS
	stwio		r7, 0(r2)
	
	cmpeqi r2, r7, 15
	bne	r2, r0, USE_PITCH15
	cmpeqi r2, r7, 14
	bne	r2, r0, USE_PITCH14
	cmpeqi r2, r7, 13
	bne	r2, r0, USE_PITCH13
	cmpeqi r2, r7, 12
	bne	r2, r0, USE_PITCH12
	cmpeqi r2, r7, 11
	bne	r2, r0, USE_PITCH11
	cmpeqi r2, r7, 10
	bne	r2, r0, USE_PITCH10
	cmpeqi r2, r7, 9
	bne	r2, r0, USE_PITCH9
	cmpeqi r2, r7, 8
	bne	r2, r0, USE_PITCH8
	cmpeqi r2, r7, 7
	bne	r2, r0, USE_PITCH7
	cmpeqi r2, r7, 6
	bne	r2, r0, USE_PITCH6
	cmpeqi r2, r7, 5
	bne	r2, r0, USE_PITCH5
	cmpeqi r2, r7, 4
	bne	r2, r0, USE_PITCH4
	cmpeqi r2, r7, 3
	bne	r2, r0, USE_PITCH3
	cmpeqi r2, r7, 2
	bne	r2, r0, USE_PITCH2
	cmpeqi r2, r7, 1
	bne	r2, r0, USE_PITCH1
	cmpeqi r2, r7, 0
	bne	r2, r0, HOLD		# for 0000 hold the current note

USE_PITCH15:
	movia	r9, pitch15
	br	RUN
USE_PITCH14:
	movia r9, pitch14
	br	RUN
USE_PITCH13:
	movia r9, pitch13
	br	RUN	
USE_PITCH12:
	movia r9, pitch12
	br	RUN	
USE_PITCH11:
	movia r9, pitch11
	br	RUN	
USE_PITCH10:
	movia r9, pitch10
	br	RUN	
USE_PITCH9:
	movia r9, pitch9
	br	RUN	
USE_PITCH8:
	movia r9, pitch8
	br	RUN	
USE_PITCH7:
	movia r9, pitch7
	br	RUN	
USE_PITCH6:
	movia r9, pitch6
	br	RUN	
USE_PITCH5:
	movia r9, pitch5
	br	RUN	
USE_PITCH4:
	movia r9, pitch4
	br	RUN	
USE_PITCH3:
	movia r9, pitch3
	br	RUN	
USE_PITCH2:
	movia r9, pitch2
	br	RUN
USE_PITCH1:
	movia r9, pitch1
	br	RUN	

RUN:
	# read the max offset from the waves.s data section
	# max offset = *( NUMSAMPLES + 4*pitch )	
	muli	r3, r7, 4
	movia	r2, NUMSAMPLES
	add		r3, r3, r2
	ldw		r11, 0(r3)				# r11 = max offset 
	slli	r11, r11, 1				# offset = 2 * num sample 

HOLD:
	# calculate next sample address
	addi	r10, r10, 2				# r10 = next offset
	add		r3, r9, r10				# current sample (r3) = pitch base addr (r9) + offset (r10)		
	
	# read next sample
	ldh 	r8, 0(r3)				# load the next sample
	slli	r8, r8, 16				# shift halfword to upper 16 bits

	movia 	r2, ADDR_AUDIODACFIFO	# write audio sample to audio codec
	stwio 	r8, 8(r2)     			# write to left fifo 
	stwio 	r8, 12(r2)				# write to right fifo
 
	ble		r10, r11, MAINLOOP		# if (offset > max offset)
	mov		r10, r0					# 	reset offset

	br MAINLOOP






